#  SLICE CLASSES FOR THERMAL DIFFUSION
#  this file is imported by program heatdiffxx.py
#  all classes have a diffuse function which calculates the heat flux
#    from the previous (outer) slice.
#  The diffuse functions finally calls the update function of the previous slice
#  so that the previous Slice updates its heat content according to 
#  the loss downstream.
# 
#---------- CLASS WEATHER -------------#

class TWeather:
  ''' Sets a temperature depending on the elapsed time'''

  def __init__(self,sIndex,wType,cycleTime,offset,averageT,swingT,windSpeed):
    self.sliceType = 0 #  = weather
    self.sIndex = sIndex # index in slice list or array. must be 0
    self.wType = int(wType) #squarewave=0 sinewave=1 readfromfile=2
    self.cycleTime = cycleTime #period in hours
    self.offset = offset #offset into period in hours
    self.averageT = averageT #average temperature C
    self.swingT = swingT #temperature excursion from average C
    self.windSpeed = windSpeed # m/s  but < 5
    self.tt = 20 #dummy temperature
    self.jperkg = 1000 #dummy joules per kilogram
    self.dummy = 1 #dummy used to complete arg list
    self.wT = 20 #place holder for temperature read from file

  def diffuse(self, elapsedTime,repRate,prevSlice):
    pass #empty function

  def surfaceHeatTransfer(self, surfaceT, repRate):
    '''returns joules per m2.K'''
    watts = self.windSpeed * 3.96 + 5.82
    joules = watts * repRate * (self.tt - surfaceT)
    return joules

  def update(self,backflow,repRate,elapsedTime): #backflow not used
    '''calculate new weather condition'''
    import math
    elapsedCycles = (elapsedTime + self.offset)/self.cycleTime
    fractionIntoCycle = elapsedCycles - int(elapsedCycles)
    if (self.wType == 0): #square wave
      if (fractionIntoCycle < 0.5):
        self.tt = self.averageT + self.swingT
      else:
        self.tt = self.averageT - self.swingT
    if (self.wType == 1): # sine wave
      self.tt = self.averageT + self.swingT * math.sin(fractionIntoCycle * 2 * math.pi) 
    if (self.wType == 2): # read temperature from file
      self.tt = self.wT  
    #self.showChanges()

  def showStatus(self):
    print 'WEATHER: ',
    #print 'slice type=', self.sliceType,
    print 'slice index=', self.sIndex,
    print 'weather type=', self.wType,
    print 'cycle time=', self.cycleTime,
    print 'offset=', self.offset,
    print 'average t =', self.averageT,
    print 'swing t =', self.swingT,
    print 'wind speed=', self.windSpeed,
    print ' '

  def showChanges(self):
    print 'WEATHER ',
    print 'T:',
    print ' %3.1f'% (self.tt)
   
# ----- end of weather class definition ------#

#---------CLASS WALL----------------#

class Wall:

  def __init__(self,sIndex,thick,density,heatCap,heatDiffCo,tture):
   self.sliceType = 1 # wall
   self.sIndex = sIndex # index in slice list or array
   self.thick = thick #m
   self.density = density #kg/m3
   self.heatCap = heatCap #heat capacity joules/kg.C
   self.heatDiffCo = heatDiffCo #thermal conductivity W.m^-1.K^-1
   self.tt = tture
   self.joules = self.tt * self.density * self.thick * self.heatCap
   self.fluxIn = 0 #joules flowing in from previous slice
   self.dummy = 0

  def diffuse(self, elapsedTime, repRate, prevSlice):
     if (prevSlice.sliceType == 0): # previous slice is weather
        #call its special heat transfer function
        #to calculate flux into this slice
        self.fluxIn = prevSlice.surfaceHeatTransfer(self.tt,repRate)
     else:
        #calc flux into slice on the basis of this slice's diffusion coeff.
        self.fluxIn = repRate * self.heatDiffCo * (prevSlice.tt - self.tt)/self.thick
     prevSlice.update(self.fluxIn,repRate,elapsedTime);  

  def update(self, backwash,repRate,elapsedTime):
      #update heat content
      self.joules += self.fluxIn #add flux flowing in from upstream
      self.joules -= backwash #subtract flux flowing out downstream
      #now recalc all variables
      self.tt = self.joules/(self.thick * self.density* self.heatCap)
      #self.showStatus() #testing

  def showStatus(self):
    print 'WALL: ',
    print 'slice type=', self.sliceType,
    print 'slice index=',    self.sIndex,
    print 'thickness=',    self.thick,
    print 'density=',    self.density
    print ' heat capacity=',    self.heatCap,
    print ' heat diff coeff=',    self.heatDiffCo,
    print 'joules=',    self.joules,
    print 'temperature=',    self.tt,
    print ' '

  def showChanges(self):
    print 'WALL slice ',
    print '%d' % self.sIndex,
    print 'T:',
    print '%3.1f'% (self.tt),
    print 'flux in:',
    print '%3.4f'% (self.joules)
 
# --------end of class wall definition-------------#

#-----CLASS ROOM-----------------#

class Room:
  
  def __init__(self,sIndex,thick,density,heatCap,joulesPerHour,heatOnHour,heatOffHour,tture):
    self.sliceType = 2 # room
    self.thick = thick
    self.density = density
    self.heatCap = heatCap
    self.tt = tture 
    self.joules = self.tt * density * thick * heatCap
    self.sIndex = sIndex # index in slice list or array
    self.surfaceConductance = 7 # inside surface conductance 7W/(m^2.K)
    self.joulesPerHour = joulesPerHour # energy released in room
    self.heatOnHour = heatOnHour # start heating at this hour of the day
    self.heatOffHour = heatOffHour # stop heating at this hour of the day
    self.iFlux = 0 # flux from internal water sources

  def diffuse(self, elapsedTime,repRate,prevSlice):
    self.fluxIn = self.surfaceConductance * repRate * (prevSlice.tt - self.tt)
    prevSlice.update(self.fluxIn,repRate,elapsedTime);  

  def update(self,backwash,repRate,elapsedTime):
    #update heat content
    #the room is the last slice, so does not expect to
    #take account of flow out downstream.
    self.joules += self.fluxIn #add flux flowing in from upstream
    hoursIntoDay = elapsedTime%24
    if (hoursIntoDay >= self.heatOnHour) and (hoursIntoDay < self.heatOffHour):
      #add internal heat
      self.joules += repRate * self.joulesPerHour
    #now recalc all variables
    self.tt = self.joules/(self.thick * self.density * self.heatCap)
    #self.showStatus() #testing

  def showStatus(self):
    print 'ROOM: ',
    print 'slice type=', self.sliceType,
    print 'slice index=',    self.sIndex,
    print 'thickness=',    self.thick,
    print 'density=',    self.density
    print 'heat capacity=',    self.heatCap,
    print 'heat input pr hr =',    self.joulesPerHour
    print 'heat on at hour =',    self.heatOnHour
    print 'heat off at hour =',    self.heatOffHour
    print 'joules=',    self.joules,
    print 'temperature=',    self.tt,
    print ' '

  def showChanges(self):
    print 'ROOM slice ',
    print '%d' % self.sIndex,
    print 'T:',
    print '%3.1f'% (self.tt),
    print 'flux in:',
    print '%3.4f'% (self.joules)
 
#-------- end of class ROOM definition---------#

# -------- end of class definitions -------------#
